@@ -35,16 +35,10 @@ use base qw(DBI::DBD::SqlEngine);
 use Carp;
 use vars qw(@ISA $VERSION $drh);
 
-$VERSION = "0.40";
+$VERSION = "0.41";
 
 $drh = undef;		# holds driver handle(s) once initialized
 
-my %accessors = (
-    get_meta   => "get_file_meta",
-    set_meta   => "set_file_meta",
-    clear_meta => "clear_file_meta",
-    );
-
 sub driver ($;$)
 {
     my ($class, $attr) = @_;
@@ -72,24 +66,6 @@ sub driver ($;$)
 
     $drh->{$class} = $class->SUPER::driver ($attr);
 
-    my $prefix = DBI->driver_prefix ($class);
-    if ($prefix) {
-	my $dbclass = $class . "::db";
-	while (my ($accessor, $funcname) = each %accessors) {
-	    my $method = $prefix . $accessor;
-	    $dbclass->can ($method) and next;
-	    my $inject = sprintf <<'EOI', $dbclass, $method, $dbclass, $funcname;
-sub %s::%s
-{
-    my $func = %s->can (q{%s});
-    goto &$func;
-    }
-EOI
-	    eval $inject;
-	    $dbclass->install_method ($method);
-	    }
-	}
-
     # XXX inject DBD::XXX::Statement unless exists
 
     return $drh->{$class};
@@ -117,42 +93,12 @@ sub dsn_quote
     my $str = shift;
     ref     $str and return "";
     defined $str or  return "";
-    $str =~ s/([;:])/\\$1/g;
+    $str =~ s/([;:\\])/\\$1/g;
     return $str;
     } # dsn_quote
 
-sub data_sources ($;$)
-{
-    my ($drh, $attr) = @_;
-    my $dir = $attr && exists $attr->{f_dir}
-	? $attr->{f_dir}
-	: File::Spec->curdir ();
-    my %attrs;
-    $attr and %attrs = %$attr;
-    delete $attrs{f_dir};
-    my $dsnextra = join ";", map { $_ . "=" . dsn_quote ($attrs{$_}) } keys %attrs;
-    my ($dirh) = Symbol::gensym ();
-    unless (opendir $dirh, $dir) {
-	$drh->set_err ($DBI::stderr, "Cannot open directory $dir: $!");
-	return;
-	}
-
-    my ($file, @dsns, %names, $driver);
-    $driver = $drh->{ImplementorClass} =~ m/^dbd\:\:([^\:]+)\:\:/i ? $1 : "File";
-
-    while (defined ($file = readdir ($dirh))) {
-	if ($^O eq "VMS") {
-	    # if on VMS then avoid warnings from catdir if you use a file
-	    # (not a dir) as the file below
-	    $file !~ m/\.dir$/oi and next;
-	    }
-	my $d = File::Spec->catdir ($dir, $file);
-	# allow current dir ... it can be a data_source too
-	$file ne File::Spec->updir () && -d $d and
-	    push @dsns, "DBI:$driver:f_dir=" . dsn_quote ($d) . ($dsnextra ? ";$dsnextra" : "");
-	}
-    return @dsns;
-    } # data_sources
+# XXX rewrite using TableConfig ...
+sub default_table_source { 'DBD::File::TableSource::FileSystem' }
 
 sub disconnect_all
 {
@@ -180,6 +126,14 @@ use Scalar::Util qw(refaddr); # in CORE since 5.7.3
 @DBD::File::db::ISA           = qw(DBI::DBD::SqlEngine::db);
 $DBD::File::db::imp_data_size = 0;
 
+sub data_sources
+{
+    my ($dbh, $attr, @other) = @_;
+    ref($attr) eq 'HASH' or $attr = {};
+    exists($attr->{f_dir}) or $attr->{f_dir} = $dbh->{f_dir};
+    return $dbh->SUPER::data_sources($attr, @other);
+}
+
 sub set_versions
 {
     my $dbh = shift;
@@ -197,8 +151,6 @@ sub init_valid_attributes
 	f_dir            => 1, # base directory
 	f_ext            => 1, # file extension
 	f_schema         => 1, # schema name
-	f_meta           => 1, # meta data for tables
-	f_meta_map       => 1, # mapping table for identifier case
 	f_lock           => 1, # Table locking mode
 	f_lockfile       => 1, # Table lockfile extension
 	f_encoding       => 1, # Encoding of the file
@@ -223,15 +175,18 @@ sub init_default_attributes
 
     # DBI::BD::SqlEngine::dr::connect will detect old-style drivers and
     # don't call twice
-    defined $phase or $phase = 0;
+    unless (defined $phase) {
+        # we have an "old" driver here
+        $phase = defined $dbh->{sql_init_phase};
+	$phase and $phase = $dbh->{sql_init_phase};
+	}
 
     if (0 == $phase) {
 	# f_ext should not be initialized
 	# f_map is deprecated (but might return)
 	$dbh->{f_dir}      = Cwd::abs_path (File::Spec->curdir ());
-	$dbh->{f_meta}     = {};
-	$dbh->{f_meta_map} = {}; # choose new name because it contains other keys
 
+	if(0) { # XXX remove block
 	# complete derived attributes, if required
 	(my $drv_class = $dbh->{ImplementorClass}) =~ s/::db$//;
 	my $drv_prefix = DBI->driver_prefix ($drv_class);
@@ -239,18 +194,6 @@ sub init_default_attributes
 	my $ro_attrs = $drv_prefix . "readonly_attrs";
 
 	my @comp_attrs = ();
-	if (exists $dbh->{$drv_prefix . "meta"}) {
-	    my $attr = $dbh->{$drv_prefix . "meta"};
-	    defined $attr and defined $dbh->{$valid_attrs} and
-		!defined $dbh->{$valid_attrs}{$attr} and
-		$dbh->{$valid_attrs}{$attr} = 1;
-
-	    my %h;
-	    tie %h, "DBD::File::TieTables", $dbh;
-	    $dbh->{$attr} = \%h;
-
-	    push @comp_attrs, "meta";
-	    }
 
 	foreach my $comp_attr (@comp_attrs) {
 	    my $attr = $drv_prefix . $comp_attr;
@@ -259,22 +202,17 @@ sub init_default_attributes
 	    defined $dbh->{$ro_attrs} and !defined $dbh->{$ro_attrs}{$attr} and
 		$dbh->{$ro_attrs}{$attr} = 1;
 	    }
+	    } # if 0
 	}
 
     return $dbh;
     } # init_default_attributes
 
-sub disconnect ($)
-{
-    %{$_[0]->{f_meta}} = ();
-    return $_[0]->SUPER::disconnect ();
-    } # disconnect
-
 sub validate_STORE_attr
 {
     my ($dbh, $attrib, $value) = @_;
 
-    if ($attrib eq "f_dir") {
+    if ($attrib eq "f_dir" && defined $value) {
 	-d $value or
 	    return $dbh->set_err ($DBI::stderr, "No such directory '$value'");
 	File::Spec->file_name_is_absolute ($value) or
@@ -295,13 +233,6 @@ sub get_f_versions
 
     my $class = $dbh->{ImplementorClass};
     $class =~ s/::db$/::Table/;
-    my (undef, $meta);
-    $table and (undef, $meta) = $class->get_table_meta ($dbh, $table, 1);
-    unless ($meta) {
-	$meta = {};
-	$class->bootstrap_table_meta ($dbh, $meta, $table);
-	}
-
     my $dver;
     my $dtype = "IO::File";
     eval {
@@ -311,303 +242,19 @@ sub get_f_versions
 	$dtype .= " ($dver)";
 	};
 
-    $meta->{f_encoding} and $dtype .= " + " . $meta->{f_encoding} . " encoding";
+    my $f_encoding;
+    if ($table) {
+	my $meta;
+	$table and (undef, $meta) = $class->get_table_meta ($dbh, $table, 1);
+	$meta and $meta->{f_encoding} and $f_encoding = $meta->{f_encoding};
+	} # if ($table)
+    $f_encoding ||= $dbh->{f_encoding};
+
+    $f_encoding and $dtype .= " + " . $f_encoding . " encoding";
 
     return sprintf "%s using %s", $dbh->{f_version}, $dtype;
     } # get_f_versions
 
-sub get_single_table_meta
-{
-    my ($dbh, $table, $attr) = @_;
-    my $meta;
-
-    $table eq "." and
-	return $dbh->FETCH ($attr);
-
-    (my $class = $dbh->{ImplementorClass}) =~ s/::db$/::Table/;
-    (undef, $meta) = $class->get_table_meta ($dbh, $table, 1);
-    $meta or croak "No such table '$table'";
-
-    # prevent creation of undef attributes
-    return $class->get_table_meta_attr ($meta, $attr);
-    } # get_single_table_meta
-
-sub get_file_meta
-{
-    my ($dbh, $table, $attr) = @_;
-
-    my $gstm = $dbh->{ImplementorClass}->can ("get_single_table_meta");
-
-    $table eq "*" and
-	$table = [ ".", keys %{$dbh->{f_meta}} ];
-    $table eq "+" and
-	$table = [ grep { m/^[_A-Za-z0-9]+$/ } keys %{$dbh->{f_meta}} ];
-    ref $table eq "Regexp" and
-	$table = [ grep { $_ =~ $table } keys %{$dbh->{f_meta}} ];
-
-    ref $table || ref $attr or
-	return &$gstm ($dbh, $table, $attr);
-
-    ref $table or $table = [ $table ];
-    ref $attr  or $attr  = [ $attr  ];
-    "ARRAY" eq ref $table or
-	croak "Invalid argument for \$table - SCALAR, Regexp or ARRAY expected but got " . ref $table;
-    "ARRAY" eq ref $attr or
-	croak "Invalid argument for \$attr - SCALAR or ARRAY expected but got " . ref $attr;
-
-    my %results;
-    foreach my $tname (@{$table}) {
-	my %tattrs;
-	foreach my $aname (@{$attr}) {
-	    $tattrs{$aname} = &$gstm ($dbh, $tname, $aname);
-	    }
-	$results{$tname} = \%tattrs;
-	}
-
-    return \%results;
-    } # get_file_meta
-
-sub set_single_table_meta
-{
-    my ($dbh, $table, $attr, $value) = @_;
-    my $meta;
-
-    $table eq "." and
-	return $dbh->STORE ($attr, $value);
-
-    (my $class = $dbh->{ImplementorClass}) =~ s/::db$/::Table/;
-    (undef, $meta) = $class->get_table_meta ($dbh, $table, 1);
-    $meta or croak "No such table '$table'";
-    $class->set_table_meta_attr ($meta, $attr, $value);
-
-    return $dbh;
-    } # set_single_table_meta
-
-sub set_file_meta
-{
-    my ($dbh, $table, $attr, $value) = @_;
-
-    my $sstm = $dbh->{ImplementorClass}->can ("set_single_table_meta");
-
-    $table eq "*" and
-	$table = [ ".", keys %{$dbh->{f_meta}} ];
-    $table eq "+" and
-	$table = [ grep { m/^[_A-Za-z0-9]+$/ } keys %{$dbh->{f_meta}} ];
-    ref ($table) eq "Regexp" and
-	$table = [ grep { $_ =~ $table } keys %{$dbh->{f_meta}} ];
-
-    ref $table || ref $attr or
-	return &$sstm ($dbh, $table, $attr, $value);
-
-    ref $table or $table = [ $table ];
-    ref $attr  or $attr  = { $attr => $value };
-    "ARRAY" eq ref $table or
-	croak "Invalid argument for \$table - SCALAR, Regexp or ARRAY expected but got " . ref $table;
-    "HASH" eq ref $attr or
-	croak "Invalid argument for \$attr - SCALAR or HASH expected but got " . ref $attr;
-
-    foreach my $tname (@{$table}) {
-	my %tattrs;
-	while (my ($aname, $aval) = each %$attr) {
-	    &$sstm ($dbh, $tname, $aname, $aval);
-	    }
-	}
-
-    return $dbh;
-    } # set_file_meta
-
-sub clear_file_meta
-{
-    my ($dbh, $table) = @_;
-
-    (my $class = $dbh->{ImplementorClass}) =~ s/::db$/::Table/;
-    my (undef, $meta) = $class->get_table_meta ($dbh, $table, 1);
-    $meta and %{$meta} = ();
-
-    return;
-    } # clear_file_meta
-
-sub get_avail_tables
-{
-    my $dbh = shift;
-
-    my @tables = $dbh->SUPER::get_avail_tables ();
-    my $dir    = $dbh->{f_dir};
-    my $dirh   = Symbol::gensym ();
-
-    unless (opendir $dirh, $dir) {
-	$dbh->set_err ($DBI::stderr, "Cannot open directory $dir: $!");
-	return @tables;
-	}
-
-    my $class = $dbh->FETCH ("ImplementorClass");
-    $class =~ s/::db$/::Table/;
-    my ($file, %names);
-    my $schema = exists $dbh->{f_schema}
-	? defined $dbh->{f_schema} && $dbh->{f_schema} ne ""
-	    ? $dbh->{f_schema} : undef
-	: eval { getpwuid ((stat $dir)[4]) }; # XXX Win32::pwent
-    my %seen;
-    while (defined ($file = readdir ($dirh))) {
-	my ($tbl, $meta) = $class->get_table_meta ($dbh, $file, 0, 0) or next; # XXX
-	# $tbl && $meta && -f $meta->{f_fqfn} or next;
-	$seen{defined $schema ? $schema : "\0"}{$tbl}++ or
-	    push @tables, [ undef, $schema, $tbl, "TABLE", "FILE" ];
-	}
-    closedir $dirh or
-	$dbh->set_err ($DBI::stderr, "Cannot close directory $dir: $!");
-
-    return @tables;
-    } # get_avail_tables
-
-# ====== Tie-Meta ==============================================================
-
-package DBD::File::TieMeta;
-
-use Carp qw(croak);
-require Tie::Hash;
-@DBD::File::TieMeta::ISA = qw(Tie::Hash);
-
-sub TIEHASH
-{
-    my ($class, $tblClass, $tblMeta) = @_;
-
-    my $self = bless ({ tblClass => $tblClass, tblMeta => $tblMeta, }, $class);
-    return $self;
-    } # new
-
-sub STORE
-{
-    my ($self, $meta_attr, $meta_val) = @_;
-
-    $self->{tblClass}->set_table_meta_attr ($self->{tblMeta}, $meta_attr, $meta_val);
-
-    return;
-    } # STORE
-
-sub FETCH
-{
-    my ($self, $meta_attr) = @_;
-
-    return $self->{tblClass}->get_table_meta_attr ($self->{tblMeta}, $meta_attr);
-    } # FETCH
-
-sub FIRSTKEY
-{
-    my $a = scalar keys %{$_[0]->{tblMeta}};
-    each %{$_[0]->{tblMeta}};
-    } # FIRSTKEY
-
-sub NEXTKEY
-{
-    each %{$_[0]->{tblMeta}};
-    } # NEXTKEY
-
-sub EXISTS
-{
-    exists $_[0]->{tblMeta}{$_[1]};
-    } # EXISTS
-
-sub DELETE
-{
-    croak "Can't delete single attributes from table meta structure";
-    } # DELETE
-
-sub CLEAR
-{
-    %{$_[0]->{tblMeta}} = ()
-    } # CLEAR
-
-sub SCALAR
-{
-    scalar %{$_[0]->{tblMeta}}
-    } # SCALAR
-
-# ====== Tie-Tables ============================================================
-
-package DBD::File::TieTables;
-
-use Carp qw(croak);
-require Tie::Hash;
-@DBD::File::TieTables::ISA = qw(Tie::Hash);
-
-sub TIEHASH
-{
-    my ($class, $dbh) = @_;
-
-    (my $tbl_class = $dbh->{ImplementorClass}) =~ s/::db$/::Table/;
-    my $self = bless ({ dbh => $dbh, tblClass => $tbl_class, }, $class);
-    return $self;
-    } # new
-
-sub STORE
-{
-    my ($self, $table, $tbl_meta) = @_;
-
-    "HASH" eq ref $tbl_meta or
-        croak "Invalid data for storing as table meta data (must be hash)";
-
-    (undef, my $meta) = $self->{tblClass}->get_table_meta ($self->{dbh}, $table, 1);
-    $meta or croak "Invalid table name '$table'";
-
-    while (my ($meta_attr, $meta_val) = each %$tbl_meta) {
-	$self->{tblClass}->set_table_meta_attr ($meta, $meta_attr, $meta_val);
-	}
-
-    return;
-    } # STORE
-
-sub FETCH
-{
-    my ($self, $table) = @_;
-
-    (undef, my $meta) = $self->{tblClass}->get_table_meta ($self->{dbh}, $table, 1);
-    $meta or croak "Invalid table name '$table'";
-
-    my %h;
-    tie %h, "DBD::File::TieMeta", $self->{tblClass}, $meta;
-
-    return \%h;
-    } # FETCH
-
-sub FIRSTKEY
-{
-    my $a = scalar keys %{$_[0]->{dbh}->{f_meta}};
-    each %{$_[0]->{dbh}->{f_meta}};
-    } # FIRSTKEY
-
-sub NEXTKEY
-{
-    each %{$_[0]->{dbh}->{f_meta}};
-    } # NEXTKEY
-
-sub EXISTS
-{
-    exists $_[0]->{dbh}->{f_meta}->{$_[1]} or
-	exists $_[0]->{dbh}->{f_meta_map}->{$_[1]};
-    } # EXISTS
-
-sub DELETE
-{
-    my ($self, $table) = @_;
-
-    (undef, my $meta) = $self->{tblClass}->get_table_meta ($self->{dbh}, $table, 1);
-    $meta or croak "Invalid table name '$table'";
-
-    delete $_[0]->{dbh}->{f_meta}->{$meta->{table_name}};
-    } # DELETE
-
-sub CLEAR
-{
-    %{$_[0]->{dbh}->{f_meta}} = ();
-    %{$_[0]->{dbh}->{f_meta_map}} = ();
-    } # CLEAR
-
-sub SCALAR
-{
-    scalar %{$_[0]->{dbh}->{f_meta}}
-    } # SCALAR
-
 # ====== STATEMENT =============================================================
 
 package DBD::File::st;
@@ -640,7 +287,7 @@ sub FETCH
 	    # fill overall_defs unless we know
 	    unless (exists $sth->{f_overall_defs} && ref $sth->{f_overall_defs}) {
 		my $all_meta =
-		    $sth->{Database}->func ("*", "table_defs", "get_file_meta");
+		    $sth->{Database}->func ("*", "table_defs", "get_sql_engine_meta");
 		while (my ($tbl, $meta) = each %$all_meta) {
 		    exists $meta->{table_defs} && ref $meta->{table_defs} or next;
 		    foreach (keys %{$meta->{table_defs}{columns}}) {
@@ -670,88 +317,222 @@ sub FETCH
     return $sth->SUPER::FETCH ($attr);
     } # FETCH
 
-# ====== SQL::STATEMENT ========================================================
+# ====== TableSource ===========================================================
 
-package DBD::File::Statement;
+package DBD::File::TableSource::FileSystem;
 
 use strict;
 use warnings;
 
-@DBD::File::Statement::ISA = qw( DBI::DBD::SqlEngine::Statement );
+use IO::Dir;
 
-sub open_table ($$$$$)
+@DBD::File::TableSource::FileSystem::ISA = 'DBI::DBD::SqlEngine::TableSource';
+
+sub data_sources
 {
-    my ($self, $data, $table, $createMode, $lockMode) = @_;
+    my ($class, $drh, $attr) = @_;
+    my $dir = $attr && exists $attr->{f_dir}
+	? $attr->{f_dir}
+	: File::Spec->curdir ();
+    defined $dir or return; # Stream-based databases do not have f_dir
+    my %attrs;
+    $attr and %attrs = %$attr;
+    delete $attrs{f_dir};
+    my $dsn_quote = $drh->{ImplementorClass}->can("dsn_quote");
+    my $dsnextra = join ";", map { $_ . "=" . &{$dsn_quote} ($attrs{$_}) } keys %attrs;
+    my $dirh = IO::Dir->new($dir);
+    unless (defined $dirh) {
+	$drh->set_err ($DBI::stderr, "Cannot open directory $dir: $!");
+	return;
+	}
 
-    my $class = ref $self;
-    $class =~ s/::Statement/::Table/;
+    my ($file, @dsns, %names, $driver);
+    $driver = $drh->{ImplementorClass} =~ m/^dbd\:\:([^\:]+)\:\:/i ? $1 : "File";
 
-    my $flags = {
-	createMode	=> $createMode,
-	lockMode	=> $lockMode,
-	};
-    $self->{command} eq "DROP" and $flags->{dropMode} = 1;
+    while (defined ($file = $dirh->read())) {
+	my $d = File::Spec->catdir ($dir, $file);
+	# allow current dir ... it can be a data_source too
+	$file ne File::Spec->updir () && -d $d and
+	    push @dsns, "DBI:$driver:f_dir=" . &{$dsn_quote} ($d) . ($dsnextra ? ";$dsnextra" : "");
+	}
+    return @dsns;
+    } # data_sources
 
-    return $class->new ($data, { table => $table }, $flags);
-    } # open_table
+sub avail_tables
+{
+    my ($self, $dbh) = @_;
 
-# ====== SQL::TABLE ============================================================
+    my $dir    = $dbh->{f_dir};
+    defined $dir or return;	# Stream based db's cannot be queried for tables
+    my $dirh = IO::Dir->new($dir);
 
-package DBD::File::Table;
+    unless (defined $dirh) {
+	$dbh->set_err ($DBI::stderr, "Cannot open directory $dir: $!");
+	return;
+	}
+
+    my $class = $dbh->FETCH ("ImplementorClass");
+    $class =~ s/::db$/::Table/;
+    my ($file, %names);
+    my $schema = exists $dbh->{f_schema}
+	? defined $dbh->{f_schema} && $dbh->{f_schema} ne ""
+	    ? $dbh->{f_schema} : undef
+	: eval { getpwuid ((stat $dir)[4]) }; # XXX Win32::pwent
+    my %seen;
+    my @tables;
+    while (defined ($file = $dirh->read ())) {
+	my ($tbl, $meta) = $class->get_table_meta ($dbh, $file, 0, 0) or next; # XXX
+	# $tbl && $meta && -f $meta->{f_fqfn} or next;
+	$seen{defined $schema ? $schema : "\0"}{$tbl}++ or
+	    push @tables, [ undef, $schema, $tbl, "TABLE", "FILE" ];
+	}
+    $dirh->close() or
+	$dbh->set_err ($DBI::stderr, "Cannot close directory $dir: $!");
+
+    return @tables;
+    }
+
+# ====== DataSource ============================================================
+
+package DBD::File::DataSource::Stream;
 
 use strict;
 use warnings;
 
-use Carp;
-require IO::File;
-require File::Basename;
-require File::Spec;
-require Cwd;
+@DBD::File::DataSource::Stream::ISA = 'DBI::DBD::SqlEngine::DataSource';
 
 # We may have a working flock () built-in but that doesn't mean that locking
 # will work on NFS (flock () may hang hard)
 my $locking = eval { flock STDOUT, 0; 1 };
 
-@DBD::File::Table::ISA = qw( DBI::DBD::SqlEngine::Table );
+use Carp;
 
-# ====== FLYWEIGHT SUPPORT =====================================================
+sub complete_table_name
+{
+    my ($self, $meta, $file, $respect_case) = @_;
+
+    my $tbl = $file;
+    if (!$respect_case and $meta->{sql_identifier_case} == 1) { # XXX SQL_IC_UPPER
+        $tbl = uc $tbl;
+	}
+    elsif (!$respect_case and $meta->{sql_identifier_case} == 2) { # XXX SQL_IC_LOWER
+        $tbl = lc $tbl;
+	}
+
+    $meta->{f_fqfn} = undef;
+    $meta->{f_fqbn} = undef;
+    $meta->{f_fqln} = undef;
+
+    $meta->{table_name} = $tbl;
+
+    return $tbl;
+    } # complete_table_name
+
+sub apply_encoding
+{
+    my ($self, $meta, $fn) = @_;
+    defined($fn) or $fn = "file handle " . fileno($meta->{fh});
+    if (my $enc = $meta->{f_encoding}) {
+	binmode $meta->{fh}, ":encoding($enc)" or
+	    croak "Failed to set encoding layer '$enc' on $fn: $!";
+	}
+    else {
+	binmode $meta->{fh} or croak "Failed to set binary mode on $fn: $!";
+	}
+    } # apply_encoding
+
+sub open_data
+{
+    my ($self, $meta, $attrs, $flags) = @_;
+
+    $flags->{dropMode} and croak "Can't drop a table in stream";
+    my $fn = "file handle " . fileno($meta->{f_file});
+
+    if ($flags->{createMode} || $flags->{lockMode}) {
+	$meta->{fh} = IO::Handle->new_from_fd( fileno($meta->{f_file} ), "w+" ) or
+	    croak "Cannot open $fn for writing: $! (" . ($!+0) . ")";
+	}
+    else {
+	$meta->{fh} = IO::Handle->new_from_fd( fileno($meta->{f_file} ), "r" ) or
+	    croak "Cannot open $fn for reading: $! (" . ($!+0) . ")";
+	}
+
+    $meta->{fh} = $meta->{f_file};
+    if ($meta->{fh}) {
+	if (my $enc = $meta->{f_encoding}) {
+	    binmode $meta->{fh}, ":encoding($enc)" or
+		croak "Failed to set encoding layer '$enc' on $fn: $!";
+	    }
+	else {
+	    binmode $meta->{fh} or croak "Failed to set binary mode on $fn: $!";
+	    }
+	} # have $meta->{$fh}
+
+    if ($self->can_flock && $meta->{fh}) {
+	my $lm = defined $flags->{f_lock}
+		      && $flags->{f_lock} =~ m/^[012]$/
+		       ? $flags->{f_lock}
+		       : $flags->{lockMode} ? 2 : 1;
+	if ($lm == 2) {
+	    flock $meta->{fh}, 2 or croak "Cannot obtain exclusive lock on $fn: $!";
+	    }
+	elsif ($lm == 1) {
+	    flock $meta->{fh}, 1 or croak "Cannot obtain shared lock on $fn: $!";
+	    }
+	# $lm = 0 is forced no locking at all
+	}
+    } # open_data
+
+sub can_flock { $locking }
+
+package DBD::File::DataSource::File;
+
+use strict;
+use warnings;
+
+@DBD::File::DataSource::File::ISA = 'DBD::File::DataSource::Stream';
+
+use Carp;
 
 my $fn_any_ext_regex = qr/\.[^.]*/;
 
-# Flyweight support for table_info
-# The functions file2table, init_table_meta, default_table_meta and
-# get_table_meta are using $self arguments for polymorphism only. The
-# must not rely on an instantiated DBD::File::Table
-sub file2table
+sub complete_table_name
 {
-    my ($self, $meta, $file, $file_is_table, $respect_case) = @_;
+    my ($self, $meta, $file, $respect_case, $file_is_table) = @_;
 
     $file eq "." || $file eq ".."	and return; # XXX would break a possible DBD::Dir
 
+    # XXX now called without proving f_fqfn first ...
     my ($ext, $req) = ("", 0);
     if ($meta->{f_ext}) {
-	($ext, my $opt) = split m/\//, $meta->{f_ext};
+	($ext, my $opt) = split m{/}, $meta->{f_ext};
 	if ($ext && $opt) {
 	    $opt =~ m/r/i and $req = 1;
 	    }
 	}
 
     # (my $tbl = $file) =~ s/$ext$//i;
-    my ($tbl, $dir, $user_spec_file);
+    my ($tbl, $basename, $dir, $fn_ext, $user_spec_file);
     if ($file_is_table and defined $meta->{f_file}) {
 	$tbl = $file;
-	($file, $dir, undef) = File::Basename::fileparse ($meta->{f_file});
+	($basename, $dir, $fn_ext) = File::Basename::fileparse ($meta->{f_file}, $fn_any_ext_regex);
+	$file = $basename . $fn_ext;
 	$user_spec_file = 1;
 	}
     else {
-	($tbl, $dir, undef) = File::Basename::fileparse ($file, $ext);
+	($basename, $dir, undef) = File::Basename::fileparse ($file, $ext);
+	$file = $tbl = $basename;
 	$user_spec_file = 0;
 	}
 
-    !$respect_case and $meta->{sql_identifier_case} == 1 and # XXX SQL_IC_UPPER
+    if (!$respect_case and $meta->{sql_identifier_case} == 1) { # XXX SQL_IC_UPPER
+        $basename = uc $basename;
         $tbl = uc $tbl;
-    !$respect_case and $meta->{sql_identifier_case} == 2 and # XXX SQL_IC_LOWER
+	}
+    elsif (!$respect_case and $meta->{sql_identifier_case} == 2) { # XXX SQL_IC_LOWER
+        $basename = lc $basename;
         $tbl = lc $tbl;
+	}
 
     my $searchdir = File::Spec->file_name_is_absolute ($dir)
 	? ($dir =~ s|/$||, $dir)
@@ -763,14 +544,15 @@ sub file2table
 	$dir = "";
 
     unless ($user_spec_file) {
-	$file_is_table and $file = "$tbl$ext";
+	$file_is_table and $file = "$basename$ext";
 
 	# Fully Qualified File Name
 	my $cmpsub;
 	if ($respect_case) {
 	    $cmpsub = sub {
 		my ($fn, undef, $sfx) = File::Basename::fileparse ($_, $fn_any_ext_regex);
-		$fn eq $tbl and
+		$sfx = '' if $^O eq 'VMS' and $sfx eq '.';  # no extension turns up as a dot
+		$fn eq $basename and
 		    return (lc $sfx eq lc $ext or !$req && !$sfx);
 		return 0;
 		}
@@ -778,136 +560,46 @@ sub file2table
 	else {
 	    $cmpsub = sub {
 		my ($fn, undef, $sfx) = File::Basename::fileparse ($_, $fn_any_ext_regex);
-		lc $fn eq lc $tbl and
+		$sfx = '' if $^O eq 'VMS' and $sfx eq '.';  # no extension turns up as a dot
+		lc $fn eq lc $basename and
 		    return (lc $sfx eq lc $ext or !$req && !$sfx);
 		return 0;
 		}
 	    }
 
-	opendir my $dh, $searchdir or croak "Can't open '$searchdir': $!";
-	my @f = sort { length $b <=> length $a } grep { &$cmpsub ($_) } readdir $dh;
+	my @f;
+	{
+	    my $dh = IO::Dir->new ($searchdir) or croak "Can't open '$searchdir': $!";
+	    @f = sort { length $b <=> length $a } grep { &$cmpsub ($_) } $dh->read();
+	    $dh->close() or croak "Can't close '$searchdir': $!";
+	    }
 	@f > 0 && @f <= 2 and $file = $f[0];
 	!$respect_case && $meta->{sql_identifier_case} == 4 and # XXX SQL_IC_MIXED
 	    ($tbl = $file) =~ s/$ext$//i;
-	closedir $dh or croak "Can't close '$searchdir': $!";
 
 	my $tmpfn = $file;
-	if ($ext and $req ) {
+	if ($ext && $req) {
             # File extension required
-            $tmpfn =~ s/$ext$//i			or  return;
+            $tmpfn =~ s/$ext$//i or return;
             }
 	}
 
     my $fqfn = File::Spec->catfile ($searchdir, $file);
-    my $fqbn = File::Spec->catfile ($searchdir, $tbl);
+    my $fqbn = File::Spec->catfile ($searchdir, $basename);
 
     $meta->{f_fqfn} = $fqfn;
     $meta->{f_fqbn} = $fqbn;
     defined $meta->{f_lockfile} && $meta->{f_lockfile} and
 	$meta->{f_fqln} = $meta->{f_fqbn} . $meta->{f_lockfile};
 
-    $dir and $tbl = File::Spec->catfile ($dir, $tbl) unless ($user_spec_file);
+    $dir && !$user_spec_file  and $tbl = File::Spec->catfile ($dir, $tbl);
     $meta->{table_name} = $tbl;
 
     return $tbl;
-    } # file2table
+    } # complete_table_name
 
-sub bootstrap_table_meta
-{
-    my ($self, $dbh, $meta, $table) = @_;
 
-    exists  $meta->{f_dir}	or $meta->{f_dir}	= $dbh->{f_dir};
-    defined $meta->{f_ext}	or $meta->{f_ext}	= $dbh->{f_ext};
-    defined $meta->{f_encoding}	or $meta->{f_encoding}	= $dbh->{f_encoding};
-    exists  $meta->{f_lock}	or $meta->{f_lock}	= $dbh->{f_lock};
-    exists  $meta->{f_lockfile}	or $meta->{f_lockfile}	= $dbh->{f_lockfile};
-    defined $meta->{f_schema}	or $meta->{f_schema}	= $dbh->{f_schema};
-    defined $meta->{sql_identifier_case} or
-        $meta->{sql_identifier_case} = $dbh->{sql_identifier_case};
-    } # bootstrap_table_meta
-
-sub init_table_meta
-{
-    my ($self, $dbh, $meta, $table) = @_;
-
-    return;
-    } # init_table_meta
-
-sub get_table_meta ($$$$;$)
-{
-    my ($self, $dbh, $table, $file_is_table, $respect_case) = @_;
-    unless (defined $respect_case) {
-	$respect_case = 0;
-	$table =~ s/^\"// and $respect_case = 1;    # handle quoted identifiers
-	$table =~ s/\"$//;
-	}
-
-    defined $dbh->{f_meta_map}{$table} and $table = $dbh->{f_meta_map}{$table};
-
-    my $meta = {};
-    defined $dbh->{f_meta}{$table} and $meta = $dbh->{f_meta}{$table};
-
-    unless ($meta->{initialized}) {
-	$self->bootstrap_table_meta ($dbh, $meta, $table);
-
-	unless (defined $meta->{f_fqfn}) {
-	    $self->file2table ($meta, $table, $file_is_table, $respect_case) or return;
-	    }
-
-	if (defined $meta->{table_name} and $table ne $meta->{table_name}) {
-	    $dbh->{f_meta_map}{$table} = $meta->{table_name};
-	    $table = $meta->{table_name};
-	    }
-
-	# now we know a bit more - let's check if user can't use consequent spelling
-	# XXX add know issue about reset sql_identifier_case here ...
-	if (defined $dbh->{f_meta}{$table} && $dbh->{f_meta}{$table}{initialized}) {
-	    $meta = $dbh->{f_meta}{$table};
-	    }
-	else {
-	    $self->init_table_meta ($dbh, $meta, $table);
-	    $meta->{initialized} = 1;
-	    $dbh->{f_meta}{$table} = $meta;
-	    }
-	}
-
-    return ($table, $meta);
-    } # get_table_meta
-
-my %reset_on_modify = (
-    f_file     => "f_fqfn",
-    f_dir      => "f_fqfn",
-    f_ext      => "f_fqfn",
-    f_lockfile => "f_fqfn", # forces new file2table call
-    );
-
-sub register_reset_on_modify
-{
-    my ($proto, $extra_resets) = @_;
-    %reset_on_modify = (%reset_on_modify, %$extra_resets);
-    return;
-    } # register_reset_on_modify
-
-sub get_table_meta_attr
-{
-    my ($class, $meta, $attrib) = @_;
-    exists $meta->{$attrib} and
-	return $meta->{$attrib};
-    return;
-    } # get_table_meta_attr
-
-sub set_table_meta_attr
-{
-    my ($class, $meta, $attrib, $value) = @_;
-    defined $reset_on_modify{$attrib} and
-	delete $meta->{$reset_on_modify{$attrib}} and
-	delete $meta->{initialized};
-    $meta->{$attrib} = $value;
-    } # set_table_meta_attr
-
-# ====== FILE OPEN =============================================================
-
-sub open_file ($$$)
+sub open_data
 {
     my ($self, $meta, $attrs, $flags) = @_;
 
@@ -928,25 +620,20 @@ sub open_file ($$$)
 		}
 	    }
 
+	$meta->{fh} = $fh;
+
 	if ($fh) {
 	    $fh->seek (0, 0) or
 		croak "Error while seeking back: $!";
-	    if (my $enc = $meta->{f_encoding}) {
-		binmode $fh, ":encoding($enc)" or
-		    croak "Failed to set encoding layer '$enc' on $fn: $!";
-		}
-	    else {
-		binmode $fh or croak "Failed to set binary mode on $fn: $!";
-		}
-	    }
 
-	$meta->{fh} = $fh;
+	    $self->apply_encoding($meta);
+	    }
 	}
     if ($meta->{f_fqln}) {
 	$fn = $meta->{f_fqln};
 	if ($flags->{createMode}) {
 	    -f $fn and
-		croak "Cannot create table lock for $attrs->{table}: Already exists";
+		croak "Cannot create table lock at '$fn' for $attrs->{table}: Already exists";
 	    $fh = IO::File->new ($fn, "a+") or
 		croak "Cannot open $fn for writing: $! (" . ($!+0) . ")";
 	    }
@@ -959,7 +646,7 @@ sub open_file ($$$)
 	$meta->{lockfh} = $fh;
 	}
 
-    if ($locking && $fh) {
+    if ($self->can_flock && $fh) {
 	my $lm = defined $flags->{f_lock}
 		      && $flags->{f_lock} =~ m/^[012]$/
 		       ? $flags->{f_lock}
@@ -972,35 +659,157 @@ sub open_file ($$$)
 	    }
 	# $lm = 0 is forced no locking at all
 	}
+    }
+
+# ====== SQL::STATEMENT ========================================================
+
+package DBD::File::Statement;
+
+use strict;
+use warnings;
+
+@DBD::File::Statement::ISA = qw( DBI::DBD::SqlEngine::Statement );
+
+# ====== SQL::TABLE ============================================================
+
+package DBD::File::Table;
+
+use strict;
+use warnings;
+
+use Carp;
+require IO::File;
+require File::Basename;
+require File::Spec;
+require Cwd;
+require Scalar::Util;
+
+@DBD::File::Table::ISA = qw( DBI::DBD::SqlEngine::Table );
+
+# ====== UTILITIES ============================================================
+
+if ( eval { require Params::Util; } )
+{
+    Params::Util->import("_HANDLE");
+}
+else
+{
+    # taken but modified from Params::Util ...
+    *_HANDLE = sub {
+	# It has to be defined, of course
+	defined $_[0] or return;
+
+	# Normal globs are considered to be file handles
+	ref $_[0] eq 'GLOB' and return $_[0];
+
+	# Check for a normal tied filehandle
+	# Side Note: 5.5.4's tied() and can() doesn't like getting undef
+	tied($_[0]) and tied($_[0])->can('TIEHANDLE') and return $_[0];
+
+	# There are no other non-object handles that we support
+	Scalar::Util::blessed($_[0]) or return;
+
+	# Check for a common base classes for conventional IO::Handle object
+	$_[0]->isa('IO::Handle') and return $_[0];
+
+	# Check for tied file handles using Tie::Handle
+	$_[0]->isa('Tie::Handle') and return $_[0];
+
+	# IO::Scalar is not a proper seekable, but it is valid is a
+	# regular file handle
+	$_[0]->isa('IO::Scalar') and return $_[0];
+
+	# Yet another special case for IO::String, which refuses (for now
+	# anyway) to become a subclass of IO::Handle.
+	$_[0]->isa('IO::String') and return $_[0];
+
+	# This is not any sort of object we know about
+	return;
+    };
+}
+
+# ====== FLYWEIGHT SUPPORT =====================================================
+
+# Flyweight support for table_info
+# The functions file2table, init_table_meta, default_table_meta and
+# get_table_meta are using $self arguments for polymorphism only. The
+# must not rely on an instantiated DBD::File::Table
+sub file2table
+{
+    my ($self, $meta, $file, $file_is_table, $respect_case) = @_;
+
+    return $meta->{sql_data_source}->complete_table_name($meta, $file, $respect_case, $file_is_table);
+    } # file2table
+
+sub bootstrap_table_meta
+{
+    my ($self, $dbh, $meta, $table, @other) = @_;
+
+    $self->SUPER::bootstrap_table_meta($dbh, $meta, $table, @other);
+
+    exists  $meta->{f_dir}	or $meta->{f_dir}	= $dbh->{f_dir};
+    defined $meta->{f_ext}	or $meta->{f_ext}	= $dbh->{f_ext};
+    defined $meta->{f_encoding}	or $meta->{f_encoding}	= $dbh->{f_encoding};
+    exists  $meta->{f_lock}	or $meta->{f_lock}	= $dbh->{f_lock};
+    exists  $meta->{f_lockfile}	or $meta->{f_lockfile}	= $dbh->{f_lockfile};
+    defined $meta->{f_schema}	or $meta->{f_schema}	= $dbh->{f_schema};
+
+    defined $meta->{f_open_file_needed} or
+	$meta->{f_open_file_needed} = $self->can ("open_file") != DBD::File::Table->can ("open_file");
+
+    defined ($meta->{sql_data_source}) or
+	$meta->{sql_data_source} = _HANDLE ($meta->{f_file})
+	                         ? 'DBD::File::DataSource::Stream'
+				 : 'DBD::File::DataSource::File';
+    } # bootstrap_table_meta
+
+sub get_table_meta ($$$$;$)
+{
+    my ($self, $dbh, $table, $file_is_table, $respect_case) = @_;
+
+    my $meta = $self->SUPER::get_table_meta($dbh, $table, $respect_case, $file_is_table);
+    $table = $meta->{table_name};
+    return unless $table;
+
+    return ($table, $meta);
+    } # get_table_meta
+
+my %reset_on_modify = (
+    f_file     => ["f_fqfn", "sql_data_source"],
+    f_dir      => "f_fqfn",
+    f_ext      => "f_fqfn",
+    f_lockfile => "f_fqfn", # forces new file2table call
+    );
+
+__PACKAGE__->register_reset_on_modify( \%reset_on_modify );
+
+my %compat_map = map { $_ => "f_$_" } qw( file ext lock lockfile );
+
+__PACKAGE__->register_compat_map( \%compat_map );
+
+# ====== DBD::File <= 0.40 compat stuff ========================================
+
+# compat to 0.38 .. 0.40 API
+sub open_file
+{
+    my ($className, $meta, $attrs, $flags) = @_;
+
+    return $className->SUPER::open_data ($meta, $attrs, $flags);
     } # open_file
 
-# ====== SQL::Eval API =========================================================
-
-sub new
+sub open_data
 {
-    my ($className, $data, $attrs, $flags) = @_;
-    my $dbh = $data->{Database};
+    my ($className, $meta, $attrs, $flags) = @_;
 
-    my ($tblnm, $meta) = $className->get_table_meta ($dbh, $attrs->{table}, 1) or
-        croak "Cannot find appropriate file for table '$attrs->{table}'";
-    $attrs->{table} = $tblnm;
+    # compat to 0.38 .. 0.40 API
+    $meta->{f_open_file_needed}
+	? $className->open_file ($meta, $attrs, $flags)
+	: $className->SUPER::open_data ($meta, $attrs, $flags);
 
-    # Being a bit dirty here, as SQL::Statement::Structure does not offer
-    # me an interface to the data I want
-    $flags->{createMode} && $data->{sql_stmt}{table_defs} and
-	$meta->{table_defs} = $data->{sql_stmt}{table_defs};
+    return;
+    } # open_data
 
-    $className->open_file ($meta, $attrs, $flags);
-
-    my $columns = {};
-    my $array   = [];
-    my $tbl     = {
-	%{$attrs},
-	meta          => $meta,
-	col_names     => $meta->{col_names} || [],
-	};
-    return $className->SUPER::new ($tbl);
-    } # new
+# ====== SQL::Eval API =========================================================
 
 sub drop ($)
 {
@@ -1012,9 +821,9 @@ sub drop ($)
     $meta->{lockfh} and $meta->{lockfh}->close ();
     undef $meta->{fh};
     undef $meta->{lockfh};
-    $meta->{f_fqfn} and unlink $meta->{f_fqfn};
-    $meta->{f_fqln} and unlink $meta->{f_fqln};
-    delete $data->{Database}{f_meta}{$self->{table}};
+    $meta->{f_fqfn} and unlink $meta->{f_fqfn}; # XXX ==> sql_data_source
+    $meta->{f_fqln} and unlink $meta->{f_fqln}; # XXX ==> sql_data_source
+    delete $data->{Database}{sql_meta}{$self->{table}};
     return 1;
     } # drop
 
@@ -1119,9 +928,9 @@ Valid after C<< $sth->execute >>; undef for Non-Select statements.
 
 =head4 NULLABLE
 
-Not really working, always returns an array ref of ones, as DBD::CSV
-does not verify input data. Valid after C<< $sth->execute >>; undef for
-non-select statements.
+Not really working, always returns an array ref of ones, except the
+affected table has been created in this session.  Valid after
+C<< $sth->execute >>; undef for non-select statements.
 
 =head3 The following DBI attributes and methods are not supported:
 
